﻿using System;
using System.Linq;
using System.Windows.Forms;

namespace PLCi_Simple_Watch_Window
{
    public partial class Form1 : Form
    {
        // timer thread triggers application to update on screen variable values every 5 ms
        static int updateRate = 5;
        public static System.Timers.Timer UpdateTimer = new System.Timers.Timer() { Interval = updateRate };
        
        public Form1()
        {
            InitializeComponent();
           
            // create new PLCi object and connect to the controller
            myPLCi PLCi = new myPLCi();
            try
            {
                PLCi.ConnectVariables();
            }
            catch (System.NullReferenceException e)
            {
                MessageBox.Show("Null reference exception caught. Timeout?");
            }
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            UpdateTimer.Start();
            UpdateTimer.Elapsed += OnTimer;
        }


        public void OnTimer(Object source, System.Timers.ElapsedEventArgs e)    //   important to know what args the event passes, must match
        {
            // when the 5 ms timer goes off, stop it...
            UpdateTimer.Stop();
            // re-read from MachineSimple
            myPLCi PLCi = new myPLCi();
            try
            {
                PLCi.ConnectVariables();
            }
            catch (System.NullReferenceException ex)
            {
                MessageBox.Show("Null reference exception caught.");
            }
            myStruct testStruct = PLCi.ReadStruct();
            // update the text boxes displaying the variables in SimpleMachine
            SetTextX(testStruct.XPos.ToString());
            SetTextY(testStruct.YPos.ToString());
            SetTextZ(testStruct.ZPos.ToString());
            SetTextFeed(testStruct.Feed.ToString());
            SetTextStart(testStruct.CycleStart.ToString());
            SetTextStop(testStruct.CycleStop.ToString());
            SetTextBool(testStruct.BoolToWrite.ToString());
            // restart the timer
            UpdateTimer.Start();
        }
        delegate void SetTextCallback(string text);

        // updates x position text box if update is needed
        private void SetTextX(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.xPosVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextX);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.xPosVal.Text = text;
            }
        }

        // updates y position text box if update is needed
        private void SetTextY(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.yPosVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextY);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.yPosVal.Text = text;
            }
        }

        // updates z position text box if update is needed
        private void SetTextZ(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.yPosVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextZ);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.zPosVal.Text = text;
            }
        }

        // updates feed rate text box if update is needed
        private void SetTextFeed(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.xPosVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextFeed);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.feedRateVal.Text = text;
            }
        }

        // updates cycle start text box if update is needed
        private void SetTextStart(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.xPosVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextStart);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.cycleStartVal.Text = text;
            }
        }

        // updates cycle stop text box if update is needed
        private void SetTextStop(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.xPosVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextStop);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.cycleStopVal.Text = text;
            }
        }

        private void SetTextBool(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (this.xPosVal.InvokeRequired)
            {
                SetTextCallback d = new SetTextCallback(SetTextBool);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                this.boolVal.Text = text;
            }
        }

        // controls what happens when "Send to controller" is clicked
        private void send_Click(object sender, EventArgs e)
        {
            myPLCi PLCi = new myPLCi();
            try
            {
                PLCi.ConnectVariables();
            }
            catch(System.NullReferenceException ex)
            {
                MessageBox.Show("Null reference exception caught.");
            }
            myStruct testStruct = PLCi.ReadStruct();
            // if there is text in the xPos box when button is clicked
            if (xPosEdit.Text.Count() > 0)
            {
               try
                {
                    // search for a double in the x position text box
                    double xPosToSend =double.Parse(xPosEdit.Text);
                    // send x position to controller
                    PLCi.WriteStruct(xPosToSend, testStruct.YPos, testStruct.ZPos, testStruct.Feed, testStruct.CycleStart, testStruct.CycleStop, testStruct.BoolToWrite);  // only send XPos here
                }
                catch(System.FormatException ex)
                {
                    Console.WriteLine("Not a valid double.");
                    MessageBox.Show("Please enter a valid double.");
                }
            }

            // if there is text in the yPos box when button is clicked
            if (yPosEdit.Text.Count() > 0)
            {
                try
                {
                    double yPosToSend = double.Parse(yPosEdit.Text);
                    PLCi.WriteStruct(testStruct.XPos, yPosToSend, testStruct.ZPos, testStruct.Feed, testStruct.CycleStart, testStruct.CycleStop, testStruct.BoolToWrite);  // only send YPos here
                }
                catch (System.FormatException ex)
                {
                    Console.WriteLine("Not a valid double.");
                    MessageBox.Show("Please enter a valid double.");
                }
            }

            // if there is text in the zPos box when button is clicked
            if (zPosEdit.Text.Count() > 0)
            {
                try
                {
                    double zPosToSend = double.Parse(zPosEdit.Text);
                    PLCi.WriteStruct(testStruct.XPos, testStruct.YPos, zPosToSend, testStruct.Feed, testStruct.CycleStart, testStruct.CycleStop, testStruct.BoolToWrite);  // only send ZPos here
                }
                catch (System.FormatException ex)
                {
                    Console.WriteLine("Not a valid double.");
                    MessageBox.Show("Please enter a valid double.");
                }
            }

            // if there is text in the feed box when button is clicked
            if (feedEdit.Text.Count() > 0)
            {
                try
                {
                    double feedToSend = double.Parse(feedEdit.Text);
                    PLCi.WriteStruct(testStruct.XPos, testStruct.YPos, testStruct.ZPos, feedToSend, testStruct.CycleStart, testStruct.CycleStop, testStruct.BoolToWrite);  // only send feed here
                }
                catch (System.FormatException ex)
                {
                    Console.WriteLine("Not a valid double.");
                    MessageBox.Show("Please enter a valid double.");
                }
            }

            // if there is text in the cycle start box when button is clicked
            if (startEdit.Text.Count() > 0)
            {
                try
                {
                    bool startToSend = bool.Parse(startEdit.Text);
                    PLCi.WriteStruct(testStruct.XPos, testStruct.YPos, testStruct.ZPos, testStruct.Feed, startToSend, testStruct.CycleStop, testStruct.BoolToWrite);  // only send start here
                }
                catch (System.FormatException ex)
                {
                    Console.WriteLine("Not a valid bool.");
                    MessageBox.Show("Please enter a valid bool.");
                }
            }

            // if there is text in the cycle stop box when button is clicked
            if (startEdit.Text.Count() > 0)
            {
                try
                {
                    bool stopToSend = bool.Parse(stopEdit.Text);
                    PLCi.WriteStruct(testStruct.XPos, testStruct.YPos, testStruct.ZPos, testStruct.Feed, testStruct.CycleStart, stopToSend, testStruct.BoolToWrite);  // only send stop here
                }
                catch (System.FormatException ex)
                {
                    Console.WriteLine("Not a valid bool.");
                    MessageBox.Show("Please enter a valid bool.");
                }
            }

            if (boolEdit.Text.Count() > 0)
            {
                try
                {
                    bool boolToSend = bool.Parse(boolEdit.Text);
                    PLCi.WriteStruct(testStruct.XPos, testStruct.YPos, testStruct.ZPos, testStruct.Feed, testStruct.CycleStart, testStruct.CycleStop, boolToSend);  // only send stop here
                }
                catch (System.FormatException ex)
                {
                    Console.WriteLine("Not a valid bool.");
                    MessageBox.Show("Please enter a valid bool.");
                }
            }

            // clear out text boxes after pressing send
            xPosEdit.Text = "";
            yPosEdit.Text = "";
            zPosEdit.Text = "";
            feedEdit.Text = "";
            startEdit.Text = "";
            stopEdit.Text = "";
            boolEdit.Text = "";

        }

        //private void writeFile_Click(object sender, EventArgs e)
        //{
        //    myPLCi PLCi = new myPLCi();
        //    try
        //    {
        //        PLCi.ConnectVariables();
        //    }
        //    catch (System.NullReferenceException ex)
        //    {
        //        MessageBox.Show("Null reference exception caught.");
        //    }
        //    // if there is text in the fileLocation box when button is clicked
        //    if (fileLocationBox.Text.Count() > 0)
        //    {
        //        try
        //        {
        //            // try to send file input to the location
        //            PLCi.fileWrite(fileInputBox.Text, fileLocationBox.Text);
        //        }
        //        catch (System.FormatException ex)
        //        {
        //            Console.WriteLine("Invalid format.");
        //        }
        //    }
        //}
    }
}
